@@ -1,6 +1,6 @@
 import unittest
 
-from pytolemaic import PyTrust
+from pytolemaic import PyTrust, help
 
 
 class TestPyTrust(unittest.TestCase):
@@ -20,3 +20,9 @@ class TestPyTrust(unittest.TestCase):
     def test_init_example_usage(self):
         example = PyTrust.print_initialization_example()
         # todo use eval?
+
+    def test_help(self):
+        supported_keys = help()
+        for key in supported_keys:
+            print('\n\n\n*** TEST KEY:', key, '***')
+            help(key=key)
